@@ -5,18 +5,20 @@
 use std::borrow::Cow;
 use std::fmt::Write;
 use std::fs::File;
+use std::io;
 use std::sync::{Arc, Mutex};
 use std::time::{Duration, Instant};
 
-use ::console::{style, StyledObject};
-use camino::Utf8Path;
+use camino::{Utf8Path, Utf8PathBuf};
+use console::{style, StyledObject};
 
 use tracing::Level;
 use tracing_subscriber::fmt::MakeWriter;
 use tracing_subscriber::prelude::*;
 
 use crate::outcome::{LabOutcome, SummaryOutcome};
-use crate::*;
+use crate::scenario::Scenario;
+use crate::{last_line, Mutant, Options, Phase, Result, ScenarioOutcome};
 
 static COPY_MESSAGE: &str = "Copy source to scratch directory";
 
@@ -39,6 +41,24 @@ impl Console {
         }
     }
 
+    pub fn walk_tree_start(&self) {
+        self.view
+            .update(|model| model.walk_tree = Some(WalkModel::default()));
+    }
+
+    pub fn walk_tree_update(&self, files_done: usize, mutants_found: usize) {
+        self.view.update(|model| {
+            *model.walk_tree.as_mut().expect("walk tree started") = WalkModel {
+                files_done,
+                mutants_found,
+            }
+        });
+    }
+
+    pub fn walk_tree_done(&self) {
+        self.view.update(|model| model.walk_tree = None);
+    }
+
     /// Update that a cargo task is starting.
     pub fn scenario_started(&self, scenario: &Scenario, log_file: &Utf8Path) {
         let start = Instant::now();
@@ -125,28 +145,28 @@ impl Console {
 
     pub fn build_dirs_finished(&self) {}
 
-    // pub fn start_copy(&self) {
-    //     self.view.update(|model| {
-    //         assert!(model.copy_model.is_none());
-    //         model.copy_model = Some(CopyModel::new());
-    //     });
-    // }
+    pub fn start_copy(&self) {
+        self.view.update(|model| {
+            assert!(model.copy_model.is_none());
+            model.copy_model = Some(CopyModel::new());
+        });
+    }
 
-    // pub fn finish_copy(&self) {
-    //     self.view.update(|model| {
-    //         model.copy_model = None;
-    //     });
-    // }
+    pub fn finish_copy(&self) {
+        self.view.update(|model| {
+            model.copy_model = None;
+        });
+    }
 
-    // pub fn copy_progress(&self, total_bytes: u64) {
-    //     self.view.update(|model| {
-    //         model
-    //             .copy_model
-    //             .as_mut()
-    //             .expect("copy in progress")
-    //             .bytes_copied(total_bytes)
-    //     });
-    // }
+    pub fn copy_progress(&self, total_bytes: u64) {
+        self.view.update(|model| {
+            model
+                .copy_model
+                .as_mut()
+                .expect("copy in progress")
+                .bytes_copied(total_bytes)
+        });
+    }
 
     /// Update that we discovered some mutants to test.
     pub fn discovered_mutants(&self, mutants: &[Mutant]) {
@@ -190,6 +210,10 @@ impl Console {
         ));
     }
 
+    pub fn clear(&self) {
+        self.view.clear()
+    }
+
     pub fn message(&self, message: &str) {
         self.view.message(message)
     }
@@ -307,6 +331,7 @@ impl io::Write for DebugLogWriter {
 /// might be concurrent activities.
 #[derive(Default)]
 struct LabModel {
+    walk_tree: Option<WalkModel>,
     copy_model: Option<CopyModel>,
     scenario_models: Vec<ScenarioModel>,
     lab_start_time: Option<Instant>,
@@ -324,7 +349,10 @@ struct LabModel {
 
 impl nutmeg::Model for LabModel {
     fn render(&mut self, width: usize) -> String {
-        let mut s = String::with_capacity(100);
+        let mut s = String::with_capacity(1024);
+        if let Some(walk_tree) = &mut self.walk_tree {
+            s += &walk_tree.render(width);
+        }
         if let Some(copy) = self.copy_model.as_mut() {
             s.push_str(&copy.render(width));
         }
@@ -417,6 +445,26 @@ impl LabModel {
     }
 }
 
+/// A Nutmeg progress model for walking the tree.
+#[derive(Default)]
+struct WalkModel {
+    files_done: usize,
+    mutants_found: usize,
+}
+
+impl nutmeg::Model for WalkModel {
+    fn render(&mut self, _width: usize) -> String {
+        if self.files_done == 0 {
+            "Scanning tree metadata...\n".to_owned()
+        } else {
+            format!(
+                "Finding mutation opportunities: {} files done, {} mutants found\n",
+                self.files_done, self.mutants_found
+            )
+        }
+    }
+}
+
 /// A Nutmeg progress model for running a single scenario.
 ///
 /// It draws the command and some description of what scenario is being tested.
@@ -530,30 +578,26 @@ pub fn style_outcome(outcome: &ScenarioOutcome) -> StyledObject<&'static str> {
     }
 }
 
-pub fn list_mutants(mutants: &[Mutant], show_diffs: bool) {
-    for mutant in mutants {
-        println!("{}", style_mutant(mutant));
-        if show_diffs {
-            println!("{}", mutant.diff());
-        }
-    }
-}
-
-fn style_mutant(mutant: &Mutant) -> String {
+pub(crate) fn style_mutant(mutant: &Mutant) -> String {
     // This is like `impl Display for Mutant`, but with colors.
     // The text content should be the same.
-    format!(
-        "{}: replace {}{}{} with {}",
-        mutant.describe_location(),
-        style(mutant.function_name()).bright().magenta(),
-        if mutant.return_type().is_empty() {
-            ""
-        } else {
-            " "
-        },
-        style(mutant.return_type()).magenta(),
-        style(mutant.replacement_text()).yellow(),
+    let mut s = String::with_capacity(200);
+    write!(
+        &mut s,
+        "{}:{}",
+        mutant.source_file.tree_relative_slashes(),
+        mutant.span.start.line,
     )
+    .unwrap();
+    s.push_str(": replace ");
+    s.push_str(&style(mutant.function_name()).bright().magenta().to_string());
+    if !mutant.return_type().is_empty() {
+        s.push(' ');
+        s.push_str(&style(mutant.return_type()).magenta().to_string());
+    }
+    s.push_str(" with ");
+    s.push_str(&style(mutant.replacement_text()).yellow().to_string());
+    s
 }
 
 fn style_elapsed_secs(since: Instant) -> String {
@@ -586,7 +630,7 @@ fn style_mb(bytes: u64) -> StyledObject<String> {
 pub fn style_scenario(scenario: &Scenario) -> Cow<'static, str> {
     match scenario {
         Scenario::Baseline => "Unmutated baseline".into(),
-        Scenario::Mutant(mutant) => console::style_mutant(mutant).into(),
+        Scenario::Mutant(mutant) => style_mutant(mutant).into(),
     }
 }
 
