@@ -4,7 +4,7 @@
 
 use std::env;
 use std::fmt::Write;
-use std::fs::{self, read_dir};
+use std::fs::{self, read_dir, read_to_string};
 use std::io::Read;
 use std::path::{Path, PathBuf};
 use std::thread::sleep;
@@ -147,13 +147,28 @@ fn uses_cargo_env_var_to_run_cargo_so_invalid_value_fails() {
 }
 
 #[test]
-fn list_diff_json_not_yet_supported() {
-    run()
-        .args(["mutants", "--list", "--json", "--diff"])
+fn list_diff_json_contains_diffs() {
+    let cmd = run()
+        .args([
+            "mutants",
+            "--list",
+            "--json",
+            "--diff",
+            "-d",
+            "testdata/tree/factorial",
+        ])
         .assert()
-        .code(1)
-        .stderr("--list --diff --json is not (yet) supported\n")
-        .stdout("");
+        .success(); // needed for lifetime
+    let out = cmd.get_output();
+    assert_eq!(String::from_utf8_lossy(&out.stderr), "");
+    println!("{}", String::from_utf8_lossy(&out.stdout));
+    let out_json = serde_json::from_slice::<serde_json::Value>(&out.stdout).unwrap();
+    let mutants_json = out_json.as_array().expect("json output is array");
+    assert_eq!(mutants_json.len(), 3);
+    assert!(mutants_json.iter().all(|e| e.as_object().unwrap()["diff"]
+        .as_str()
+        .unwrap()
+        .contains("--- src/bin/factorial.rs")));
 }
 
 /// Return paths to all testdata trees, in order, excluding leftover git
@@ -457,6 +472,30 @@ fn list_files_json_workspace() {
         .assert_insta("list_files_json_workspace");
 }
 
+#[test]
+fn list_files_as_json_in_workspace_subdir() {
+    run()
+        .args(["mutants", "--list-files", "--json"])
+        .current_dir("testdata/tree/workspace/main2")
+        .assert()
+        .stdout(indoc! {r#"
+            [
+              {
+                "package": "cargo_mutants_testdata_workspace_utils",
+                "path": "utils/src/lib.rs"
+              },
+              {
+                "package": "main",
+                "path": "main/src/main.rs"
+              },
+              {
+                "package": "main2",
+                "path": "main2/src/main.rs"
+              }
+            ]
+        "#});
+}
+
 #[test]
 fn workspace_tree_is_well_tested() {
     let tmp_src_dir = copy_of_testdata("workspace");
@@ -484,13 +523,20 @@ fn workspace_tree_is_well_tested() {
         assert_eq!(baseline_phases.len(), 2);
         assert_eq!(baseline_phases[0]["process_status"], "Success");
         assert_eq!(
-            baseline_phases[0]["argv"].as_array().unwrap()[1..],
-            ["build", "--tests", "--workspace"]
+            baseline_phases[0]["argv"].as_array().unwrap().into_iter().map(|v| v.as_str().unwrap()).skip(1).collect_vec().join(" "),
+            "build --tests --package cargo_mutants_testdata_workspace_utils --package main --package main2"
         );
         assert_eq!(baseline_phases[1]["process_status"], "Success");
         assert_eq!(
-            baseline_phases[1]["argv"].as_array().unwrap()[1..],
-            ["test", "--workspace"]
+            baseline_phases[1]["argv"]
+                .as_array()
+                .unwrap()
+                .into_iter()
+                .map(|v| v.as_str().unwrap())
+                .skip(1)
+                .collect_vec()
+                .join(" "),
+            "test --package cargo_mutants_testdata_workspace_utils --package main --package main2"
         );
     }
 
@@ -521,17 +567,52 @@ fn workspace_tree_is_well_tested() {
         assert_eq!(baseline_phases.len(), 2);
         assert_eq!(baseline_phases[0]["process_status"], "Success");
         assert_eq!(
-            baseline_phases[0]["argv"].as_array().unwrap()[1..],
-            ["build", "--tests", "--workspace"]
+            baseline_phases[0]["argv"].as_array().unwrap()[1..].into_iter().map(|v| v.as_str().unwrap()).join(" "),
+            "build --tests --package cargo_mutants_testdata_workspace_utils --package main --package main2",
         );
         assert_eq!(baseline_phases[1]["process_status"], "Success");
         assert_eq!(
-            baseline_phases[1]["argv"].as_array().unwrap()[1..],
-            ["test", "--workspace"]
+            baseline_phases[1]["argv"].as_array().unwrap()[1..]
+                .into_iter()
+                .map(|v| v.as_str().unwrap())
+                .join(" "),
+            "test --package cargo_mutants_testdata_workspace_utils --package main --package main2",
         );
     }
 }
 
+#[test]
+/// Baseline tests in a workspace only test the packages that will later
+/// be mutated.
+/// See <https://github.com/sourcefrog/cargo-mutants/issues/151>
+fn in_workspace_only_relevant_packages_included_in_baseline_tests() {
+    let tmp = copy_of_testdata("package_fails");
+    run()
+        .args(["mutants", "-f", "passing/src/lib.rs", "--no-shuffle", "-d"])
+        .arg(tmp.path())
+        .assert()
+        .success();
+    assert_eq!(
+        read_to_string(tmp.path().join("mutants.out/caught.txt")).unwrap(),
+        indoc! { "\
+            passing/src/lib.rs:1: replace triple -> usize with 0
+            passing/src/lib.rs:1: replace triple -> usize with 1
+            "}
+    );
+    assert_eq!(
+        read_to_string(tmp.path().join("mutants.out/timeout.txt")).unwrap(),
+        ""
+    );
+    assert_eq!(
+        read_to_string(tmp.path().join("mutants.out/missed.txt")).unwrap(),
+        ""
+    );
+    assert_eq!(
+        read_to_string(tmp.path().join("mutants.out/unviable.txt")).unwrap(),
+        ""
+    );
+}
+
 #[test]
 fn copy_testdata_doesnt_include_build_artifacts() {
     // If there is a target or mutants.out in the source directory, we don't want it in the copy,
@@ -622,8 +703,8 @@ fn well_tested_tree_quiet() {
         fs::read_to_string(tmp_src_dir.path().join("mutants.out/outcomes.json")).unwrap();
     println!("outcomes.json:\n{outcomes_json}");
     let outcomes: serde_json::Value = outcomes_json.parse().unwrap();
-    assert_eq!(outcomes["total_mutants"], 27);
-    assert_eq!(outcomes["caught"], 27);
+    assert_eq!(outcomes["total_mutants"], 38);
+    assert_eq!(outcomes["caught"], 38);
     assert_eq!(outcomes["unviable"], 0);
     assert_eq!(outcomes["missed"], 0);
 }
@@ -842,11 +923,11 @@ fn small_well_tested_mutants_with_cargo_arg_release() {
     println!("{}", baseline_log_path.display());
     let log_content = fs::read_to_string(baseline_log_path).unwrap();
     println!("{log_content}");
-    regex::Regex::new(r"cargo.* build --tests --workspace --release")
+    regex::Regex::new(r"cargo.* build --tests --manifest-path .* --release")
         .unwrap()
         .captures(&log_content)
         .unwrap();
-    regex::Regex::new(r"cargo.* test --workspace --release")
+    regex::Regex::new(r"cargo.* test --manifest-path .* --release")
         .unwrap()
         .captures(&log_content)
         .unwrap();
@@ -935,21 +1016,19 @@ fn already_failing_tests_are_detected_before_running_mutants() {
         .assert()
         .code(4)
         .stdout(
-            predicate::str::contains("running 1 test\ntest test_factorial ... FAILED").normalize(),
-        )
-        .stdout(
-            predicate::str::contains(
-                "thread 'test_factorial' panicked at 'assertion failed: `(left == right)`
-  left: `720`,
- right: `72`'",
-            )
-            .normalize(),
-        )
-        .stdout(predicate::str::contains("lib.rs:11:5"))
-        .stdout(predicate::str::contains(
-            "cargo test failed in an unmutated tree, so no mutants were tested",
-        ))
-        .stdout(predicate::str::contains("test result: FAILED. 0 passed; 1 failed;").normalize());
+            predicate::str::contains("running 1 test\ntest test_factorial ... FAILED")
+                .normalize()
+                .and(predicate::str::contains("thread 'test_factorial' panicked"))
+                .and(predicate::str::contains("72")) // the failing value should be in the output
+                .and(predicate::str::contains("lib.rs:11:5"))
+                .and(predicate::str::contains(
+                    "cargo test failed in an unmutated tree, so no mutants were tested",
+                ))
+                .and(
+                    predicate::str::contains("test result: FAILED. 0 passed; 1 failed;")
+                        .normalize(),
+                ),
+        );
 }
 
 #[test]
